clear;
example = 1;

%
% Example 1 from Beard, Babson, Curtis, Qian:
%
%   Test script for function 'cycles'. Expect to find
%   13 cyles in matrix S, which represents a 8-reaction
%   network of 4 species:
% 
%       B ---r3---D---r7--- 
%       |  \      |
%       r2  r1    r4
%       |      \  |
% --r6- A ---r5---C---r8--- 
%  
    if example == 1;
    %        s1  s2  s3  s4  s5  s6  s7  s8    
        S = [ 0  -1   0   0  -1  +1   0   0
             -1  +1  -1   0   0   0   0   0
             +1   0   0  +1  +1   0   0  -1
              0   0  +1  -1   0   0  -1   0 ];   
        ext_flux = [1 1 1 ]; 
        C = cycles(S);         
    end   
%
% Example 2    
    if example == 2;
       inputfile = 'ecoli_revision.txt';
       [S, C, species, enzymes] = read_data(inputfile);     
       ext_flux = [  1 1 1 1 ];    
    end
    
%  
%   %------------------------------------------------------%
%   | Initization and extract basis from Cycle matrix      |
%   %------------------------------------------------------%
%     
    [m,n_all] = size(S);                 % m is the number of reactants
                                         % n_all is total number of fluxes                                         
    n_ext = length(ext_flux);            % the number of external fluxes. 
    n_int = n_all - n_ext;               % the number of internal fluxes.
       
    [C_in C_ex] = extract_basis(C,ext_flux);    
    nc_ex = size(C_ex,2);                % number of columns of external cycle. 
    nc_in = size(C_in,2);                % number of columns of internal cycle.    
%  
%   %------------------------------------------------------%
%   | Optimization method to predict the flux direction    |
%   %------------------------------------------------------%
%     
    if example == 1   % for example 1 only, it's too expensive for example 2
                      % if using this method.
                      
    LB = zeros(nc_ex,1); 
    UB = +10.*ones(nc_ex,1);             %UB is an array of arbitrary finite 
                                         %positive real numbers. 

    % A feasible initial guess for calculations below:
    xo = ones(nc_ex,1); 
    for i = 1:n_int 
         max_min= -1;
         [x,F,xflag(i,1)] = fmincon(@flux,xo,-C_ex(n_int+1:n_all,:),... 
             zeros(n_ext,1),[],[],LB,UB,@T_constraint,[],C_in,C_ex,max_min,i);         
         J_min(i) = F;  

         max_min= 1;     
         [x,F,xflag(i,2)] = fmincon(@flux,xo,-C_ex(n_int+1:n_all,:),... 
             zeros(n_ext,1),[],[],LB,UB,@T_constraint,[],C_in,C_ex,max_min,i);         
         J_max(i) = -F; 
    end 
    
    if sum(sum( xflag<0)) ~= 0 
          disp('error in optimization'); 
          xflag 
    end 

    [(1:n_int)' J_min' J_max' ]
    
    end   % only for example 1.
    
%  
%   %------------------------------------------------------%
%   | Enumeration method to predict the flux direction     |
%   %------------------------------------------------------%

    Sign_Pattern = sign_pattern(S, C_ex, ext_flux);    
    n = length(Sign_Pattern);   
    [(1:n)'   Sign_Pattern]        

    %have to check T-constraint for generated sign pattern.
    k = 1;
    match = [];
    for j = 1:nc_in
        for i = 1:n_all
            if C_in(i,j) ~= 0 &&  Sign_Pattern(i) ~= -11 
                 if C_in(i,j)*Sign_Pattern(i) < 0
                    match(k) = 1;
                 else
                    match(k) = -1;
                 end
                 k = k + 1;
            end
        end
        if length(match) > 0 && abs(sum(match)) == sum(ones(length(match),1))
            cycle(j) = 1;
            disp('found a possible cycle in sign pattern');
        else
            cycle(j) = -1;
        end        
    end

    cycle
 
  
