
%\BeginDoc
%
%\Name: T_constraint
%
%\Description: 
%     Checks to see if a flux violates the thermoynamic
%     constraint in terms of the matroid formalism.
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%
%\References:
%  1. Yang F, Qian H, Beard DA: Ab Initio Prediction of Thermodynamically 
%     Feasible Reaction Directions from Biochemical Network Stoichiometry.
%     Metabolic Engineering, in review.
%  2. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  3. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  4. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical net-works. J Theor Biol 2004, 228:327-333.
%
%\EndDoc
%
%-------------------------------------------------------------------------
%
%\Arguments
%   J         Putative flux vector.
%   C         matrix of cycles
%   j_obj     placeholder not used in this function
%
%\Program outputs:
%   con_ie    vector of inequality constraints.
%   con_eq    vector of equality constraints
%
function [con_ie,con_eq] = T_constraint(x,C_in,C_ex,max_min,j_obj);

J = C_ex*x;
n_int = length(C_in(:,1));
con_ie = [abs(sign(C_in')*sign(J(1:n_int))) - sum(C_in~=0)' + 0.5];
con_eq = [];
