%\BeginDoc
%
%\Name: extract_basis 
%
%\Description: 
%   This code extract two cycle matrices from the matroid cycles.
%
%\Arguments
%   C_all      Matrix stored matroid cycles, including external fluxes. 
%
%   ext_flux   Integer vector, indicates the external flux directions.
%              Positive direction was denoted as '+1' and negative as '-1'.
%              If there is no flux, set it to '0'.
%
%\Program outputs:
%   C_in       the internal cycles. 
%   C_ex       cycles with imposed sign constraints and without cycles 
%              belonging to C_in. 
%             
%\EndDoc
%
%-------------------------------------------------------------------------    
%
    function [C_in C_ex] = extract_basis(C_all,ext_flux)
%  
%   %--------------------%
%   | Initialization.    |
%   %--------------------%
%    
    C_in = [];
    C_ex = [];
    [n_all,n] = size(C_all);      
    
    n_ext = length(ext_flux);     
    n_int = n_all - n_ext;
%  
%   %------------------------------------------------------%
%   | Main loop to extract the basis according to the      |
%   | external flux direction.                             |
%   %------------------------------------------------------%
%    
%   Loop for each column in C_all.
    i = 1; while i <= n & n_int ~= n_all 
      v0 = sign(C_all(n_int+1:n_all,i));       
%  
%     %-------------------------------------------------------------%
%     | Initialize local variables and exit if it's a zero vector.  |
%     %-------------------------------------------------------------%
%
      add = 1; ifind = 0; same_sign = 0;      
      if sum(v0~=0) == 0
           C_in = [C_in C_all(:,i)];
           add = 0;
      end        
%  
%     %-------------------------------------------------------------%
%     | Find whether they are in the same sign wrt ext_flux.        |
%     | Then check their sign pattern consistancy.                  |
%     %-------------------------------------------------------------%
% 
      j = 1; while j <= length(v0) & add == 1 
          if ifind == 0
              if ext_flux(j) == 0 & v0(j) ~= 0 
                  add = 0;
              elseif ext_flux(j) * v0(j) > 0 
                  same_sign =  1;   ifind = 1;                  
              elseif ext_flux(j) * v0(j) < 0 
                  same_sign = -1;   ifind = 1;                 
              end 
          else
              if ext_flux(j) == 0 & v0(j) ~= 0 
                  add = 0;              
              elseif ext_flux(j) * v0(j) > 0 & same_sign == -1
                  add = 0;
              elseif ext_flux(j) * v0(j) < 0 & same_sign == 1
                  add = 0;
              end
          end
          j = j + 1;
      end             
%  
%     %-----------------------------------%
%     | Add into C_in C_ex matrix.        |
%     %-----------------------------------%
%
      if add == 1 & same_sign == -1    % Change the sign if necesary. 
         C_all(:,i) = -1 .* C_all(:,i); 
      end
           
      if add == 1                      % Add into N.       
         C_ex = [C_ex C_all(:,i)];             
      end
      
    i = i+1;  end  % End of the loop         
%    
%  %----------------------------------%
%  | End of extract_basis function    |
%  %----------------------------------%
%  
