%\BeginDoc
%
%\Name: flux
%
%\Description: 
%     An objective function, minimize or maximize one particular flux.
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%
%\References:
%  1. Yang F, Qian H, Beard DA: Ab Initio Prediction of Thermodynamically 
%     Feasible Reaction Directions from Biochemical Network Stoichiometry.
%     Metabolic Engineering, in review.
%  2. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  3. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  4. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical net-works. J Theor Biol 2004, 228:327-333.
%
%\EndDoc
%
%-------------------------------------------------------------------------
%
%\Arguments
%   x          A vector with dimension of size(C_ex,2). 
%              J is a linear combination of all cycles in C_ex.
%
%\Program outputs:
%   fo         flux value, scalar.
%
 

function [fo] = flux(x,C_in,C_ex,max_min,j_obj);

J = C_ex*x;

if max_min == 1          %looking for maximum.
    fo = -J(j_obj);
elseif max_min == -1     %looking for minimum.
    fo = J(j_obj);    
end


