%\BeginDoc
%
%\Name: read_data
%
%\Description: 
%     An interface to metatool program, meta4.3_double.exe. 
%       1). Reading enzymes, species, reaction information.
%       2). Calculating minimal cycle basis.
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%
%\References:
%  1. Yang F, Qian H, Beard DA: Ab Initio Prediction of Thermodynamically 
%     Feasible Reaction Directions from Biochemical Network Stoichiometry.
%     Metabolic Engineering, in review.
%  2. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  3. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  4. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical net-works. J Theor Biol 2004, 228:327-333.
%
%\EndDoc
%
%-------------------------------------------------------------------------
%
%\Arguments
%   inputfile  A txt file containing the network information. 
%
%\Program outputs:
%   S         Stochiometric matrix. 
%   C         Minimal cycle basis. 
%   species   All reactants in the network. 
%   enzymes   All enzymes in the network. 
%

function [S, C, species, enzymes] = read_data(inputfile)
            
            
    species = [];
    enzymes = [];
    genes = [];
    reversibility = [];
    C = [];
    S = [];    
  
    sfile = textread(inputfile,'%s','commentstyle', 'matlab' );
    maximum = length(sfile);
    
    readreactions = 0;
    startflag = 0;
    index = 0;
    while ~(startflag)
      index = index + 1;
      string_i = sfile(index,:);
      if strcmp(string_i,'-CAT') 
        startflag = 1;
      end
    end

    stopflag = 0;
    rxn_num = 0;
    while ~(stopflag)

      rxn_num = rxn_num + 1;
      rxn_name = [];

    % (1) Read reaction name    
      readrxn = 1;
      while readrxn
        index = index + 1;
        string_i = char(sfile(index,:));
        string_i = [' ', string_i];
        
      
        
        if strcmp(sfile(index+1,:),':')             
          readrxn = 0;
        end
        rxn_name = [rxn_name string_i];        
      end
      enzymes = [enzymes
                 cellstr(rxn_name)];      

    % (2) Read gene name
      index = index + 1;
      %genes = [genes
      %          sfile(index,:)];

    % (3) Read reaction substrates
      readsub = 1;
      while readsub     
        index = index + 1;
        string_i = sfile(index,:);
        

        
    % Get species name and stoichiometric number for current species:
        if isnumeric(str2num(strvcat(string_i)))*size(str2num(strvcat(string_i)))
          s = str2num(char(string_i));
          index = index + 1;
          species_i = sfile(index,:);
        elseif strcmp(string_i, '=')      
            readsub = 0;
            continue;
        else
          s = 1;
          species_i = string_i;
        end
    % Check to see if species is in list
        [n,m] = size(species);
        new_species = 1;
        for i = 1:n
          if strcmp(species_i,species(i,:))
             new_species = 0;
             species_num = i;
          end
        end
        if new_species
          species = [species
                     species_i];
          species_num = n+1;
        end
    % Update S matrix
        S(species_num,rxn_num) = -s;

        index = index + 1;
        string_i = sfile(index,:);
        if strcmp(string_i,'=')
          readsub = 0;
          reversibility(rxn_num) = 1;
        elseif strcmp(string_i,'>')
          readsub = 0;
          reversibility(rxn_num) = 0;
        end   
      end 

     % (4) Read reaction products
      readsub = 1;
      while readsub     
        index = index + 1;
        string_i = sfile(index,:);       
    % Get species name and stoichiometric number for current species:                 
        if isnumeric(str2num(strvcat(string_i)))*size(str2num(strvcat(string_i)))
          s = str2num(char(string_i));
          index = index + 1;
          species_i = sfile(index,:);
        elseif strncmp(string_i, '.', 1)
            readsub =0;
            continue;
        else     
          s = 1;
          species_i = string_i;
        end

    % Check to see if species is in list
        [n,m] = size(species);
        new_species = 1;
        for i = 1:n
          if strcmp(species_i,species(i,:))
             new_species = 0;
             species_num = i;
          end
        end
        if new_species
          species = [species
                     species_i];
          species_num = n+1;
        end
    % Update S matrix
        S(species_num,rxn_num) = +s;

        index = index + 1;
        string_i = sfile(index,:);
        clist =  char(string_i);
        if strcmp(clist(1),'.')
          readsub = 0;
        end   
      end 

  

      if index >= maximum 
        stopflag = 1;
      end

    end
        
     S1 = S;
              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
     
    species; 
    enzymes; 
    

    boundary_enzymes = [];
     for i = 1: length(enzymes)
         if length(char(cellstr(enzymes{i}))) == 6 
             boundary_enzymes = [boundary_enzymes
                                 cellstr(enzymes{i})];
         end
     end
     
     n_ext = length(boundary_enzymes) ;                   
             
     n_enzymes = length(enzymes) - length(boundary_enzymes);
    
    
    iauto =0;
    if iauto == 1
%    clear file content    
    fid2 = fopen('tmp1','w');
    fclose(fid2);
    
    fid2=fopen('tmp1','w');
    fid =fopen(inputfile);    
    
    ifind = 0;
    istart = 0; 
        while ~feof(fid)   
                            
            tline = fgetl(fid);   
            if  length(findstr('-ENZREV',tline)) == 1 
                 count = fprintf(fid2, '\n%s', '-ENZREV');
                 for i = 1:n_enzymes 
                    count = fprintf(fid2, '\n%s', enzymes{i});
                 end
             elseif length(findstr('-METINT',tline)) == 1 
                 count = fprintf(fid2, '\n%s', '-METINT');
                 for i = 1:length(species) 
                    count = fprintf(fid2, '\n%s', species{i});
                 end
             elseif length(findstr('-ENZIRREV',tline)) == 1
                 count = fprintf(fid2, '\n%s', '-ENZIRREV');
                 for i = 1:length(boundary_enzymes) 
                    count = fprintf(fid2, '\n%s', boundary_enzymes{i});
                 end
             else
                 count = fprintf(fid2, '\n%s', tline);
             end
             
        end
        %count = fprintf(fid2, '\n%s', '   ');  
        fclose(fid);    fclose(fid2);       
        
        inputfile = 'tmp1' ;  
    end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
idata = 1;
if idata == 1 
    command = strcat(['meta4.3_double.exe', '  ', inputfile, '  ',  'tmp2 &']);
    [s, w] = dos(command);      
    if s ~= 0 
      %  pause;
    end

    fid=fopen('tmp2'); 
    ifind = 0;
    istart = 0;        
% read STOICHIOMETRIC MATRIX.
        while 1
           tline = fgetl(fid);   
 
           if istart == 0
                if length(tline) > 10 & length(findstr('STOICHIOMETRIC MATRIX',tline)) == 1 
                    ifind = 1;
                end

                if ifind == 1 & length(tline) > 10 ...
                    if length(findstr('matrix dimension',tline)) == 1                
                    [a, b, c, d, e] = strread(tline,'%s%s%s%s%s','delimiter',' ');
                    a = char(a); b = char(b); c = char(c); d = char(d); e = char(e); 
                    rows = str2num(c(2:length(c)));
                    cols = str2num(e(2:length(e)));
                    istart = 1; 
                    end;
                    break;
                end
           end
        end
        
        if istart
           for i = 1:rows
               tline = fgetl(fid);                
               S(i,1:cols) = sscanf(tline, '%f');
           end
        end
        
        
        if abs(sum(sum(S1-S))) ~= 0 
            disp( 'STOICHIOMETRIC MATRIX ERROR');
            pause
        end
                
%   read elementary modes        
        ifind = 0;
        istart = 0;        
        while 1
           tline = fgetl(fid);  
           
           if istart == 0
                if ifind == 1 & length(tline) > 1                     
                  if length(findstr('matrix dimension',tline)) == 1               
                    [a, b, c, d, e] = strread(tline,'%s%s%s%s%s','delimiter',' ');
                    a = char(a); b = char(b); c = char(c); d = char(d); e = char(e); 
                    rows = str2num(c(2:length(c)));
                    cols = str2num(e(2:length(e)));
                    istart = 1; 
                    break;
                  else
                    break;
                  end
                end                  
                  
                if length(tline) > 10 & length(findstr('ELEMENTARY MODES',tline)) == 1
                    ifind = 1;
                end
                
                

           end
        end
        
        if istart
           for i = 1:rows
               tline = fgetl(fid);         
               temp = sscanf(tline, '%f')';
               if size(temp, 2) == cols
                   C(i,1:cols) = temp;
               elseif size(temp, 1) == cols
                   C(i,1:cols) = temp';
               end
               
           end
        end
        
        C = C';
        fclose(fid);
        
   command1 = strcat(['del', '  ',  'fna* ']);        
   %command1 = strcat(['del', '  ', 'tmp1 ', '  ', 'tmp2 ', '  ', 'fna* ']);
   [s, w] = dos(command1, '-echo');  
end   


