%\BeginDoc
%
%\Name: sign_pattern
%
%\Description: 
%     This code shows how to predict the direction of each steady-state flux
%     that satisfies both mass balance constraints and thermodynamic
%     constraints. 
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%
%\Arguments
%   S          Stoichiometric matrix; containts the stoichiometric numbers
%              for reactions in a reaction network. (integer or double)
%
%   C_ex       Cycle basis matrix, imposed boundary flux direction, and 
%              excluding internal cycle basis. (integer or double)
%
%   ext_flux   Integer vector, indicating the external flux directions.
%              Positive direction was denoted as '+1' and negative as '-1'.
%              If no flux exists, set it to '0'.
%
%\Program outputs:
%   N          Vector stored the predicted flux directions. 
%              '-11' denote reversible, '-1' denote backward direction and 
%              '+1' deonoe forward direction. 
%             
%\EndDoc
%
%-------------------------------------------------------------------------

    function [N] = sign_pattern(S,C_ex, ext_flux);

%   Initialize some local variables.
    n_all = size(S,2);              % denotes the # of all reactions    
    n_int = n_all-length(ext_flux); % denotes the # of internal reactions.
    
%   Return if without external flux.
    if n_all == n_int
        N = [];
        return
    end
    
    N = C_ex;
%
%   %------------------------------------------------------%
%   | Generate the sign pattern from the N matrix.         |
%   %------------------------------------------------------%
%    
    [m, n] = size(N);  
    SignOfReaction = [];
    
    i = 1;  while i <= m
        itest = 1;
        j = 1;   while j <= n        
            if itest == 1 
                if sign(N(i,j)) == 0 
                    j = j+1;
                    sor = 0;
                    continue;
                elseif sign(N(i,j)) > 0 
                    sor = 1;
                    itest = 0;
                elseif sign(N(i,j)) < 0
                    sor = -1;
                    itest = 0;
                end
            else
                if  sign(N(i,j)) == 0 
                    j = j+1;
                    continue;
                elseif sor > 0 & N(i,j) > 0 
                    sor = 1;
                elseif sor < 0 & N(i,j) < 0 
                    sor = -1;
                elseif (sor > 0 & N(i,j) < 0) | (sor < 0 &  N(i,j) > 0) 
                    sor = -11;
                    j = j + 1;
                    break;
                end
            end
            j = j+1;
        end
        SignOfReaction(i) = sor;          
        i = i + 1;
    end
    
    N =  SignOfReaction(1:m)';